IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Manufacturer_GetAllManufacturers]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Manufacturer_GetAllManufacturers]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Manufacturer_GetAllManufacturers]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Manufacturer_GetAllManufacturers]
GO
/****** Object:  StoredProcedure [FNS_Manufacturer_GetAllManufacturers]    Script Date: 04/04/2013 11:42:28 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Manufacturer_GetAllManufacturers]
(
	@manufacturerName nvarchar(MAX) ,	--Manufacturer name
	@StoreId int, -- Store mapping
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@PageIndex			int , --Page index
	@PageSize			int , --Page size
	@ShowHidden			bit, --A value indicating whether to show hidden records
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit, --A value indicating whether to Ignore Store Limitations
	@TotalRecords		int OUTPUT
)
AS
BEGIN
SET NOCOUNT ON

set @manufacturerName=ltrim(rtrim(isnull(@manufacturerName,'')))

--paging
DECLARE @PageLowerBound int
DECLARE @PageUpperBound int
DECLARE @RowsToReturn int
SET @RowsToReturn = @PageSize * (@PageIndex + 1)	
SET @PageLowerBound = @PageSize * @PageIndex
SET @PageUpperBound = @PageLowerBound + @PageSize + 1

CREATE TABLE #DisplayOrderTmp 
(
	[Id] int IDENTITY (1, 1) NOT NULL,
	[ManufacturerId] int NOT NULL
)

if @ShowHidden=1
begin
	insert into #DisplayOrderTmp (ManufacturerId)
	select M.Id  
	from Manufacturer M WITH(NOLOCK)
	where (len(@manufacturerName)=0 or M.Name like '%'+@manufacturerName+'%')
		and M.Deleted=0
	Order by M.DisplayOrder, M.Id				
	
end
else
begin
	--filter by customer role IDs (access control list)
	SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
	CREATE TABLE #FilteredCustomerRoleIds
	(
		CustomerRoleId int not null
	)
	INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

	insert into #DisplayOrderTmp (ManufacturerId)
	select M.Id  
	from Manufacturer M WITH(NOLOCK)
	where (len(@manufacturerName)=0 or M.Name like '%'+@manufacturerName+'%')
		and M.Deleted=0
		and M.Published=1
		and (@IgnoreAcl=1 OR M.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl
									WHERE [acl].EntityId = M.Id AND [acl].EntityName = 'Manufacturer'
								)
							))
		and (@StoreId=0 or @IgnoreStoreLimitations=1 or M.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
			where sm.EntityName = 'Manufacturer' and sm.StoreId=@StoreId and sm.EntityId=M.Id))

	Order by M.DisplayOrder, M.Id					

	drop table #FilteredCustomerRoleIds
end
				
	CREATE TABLE #PageIndex 
	(
		[IndexId] int IDENTITY (1, 1) NOT NULL,
		[ManufacturerId] int NOT NULL
	)
	INSERT INTO #PageIndex ([ManufacturerId])
	SELECT [ManufacturerId]
	FROM #DisplayOrderTmp
	GROUP BY [ManufacturerId]
	ORDER BY min([Id])

	--total records
	SET @TotalRecords = @@rowcount
	
	DROP TABLE #DisplayOrderTmp

	--return categories
	SELECT TOP (@RowsToReturn)
		C.*
	FROM
		#PageIndex [pi]
		INNER JOIN Manufacturer C on C.Id = [pi].[ManufacturerId]
	WHERE
		[pi].IndexId > @PageLowerBound AND 
		[pi].IndexId < @PageUpperBound
	ORDER BY
		[pi].IndexId
	
	DROP TABLE #PageIndex
END

GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Manufacturer_GetById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Manufacturer_GetById]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Manufacturer_GetById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Manufacturer_GetById]
GO
/****** Object:  StoredProcedure [FNS_Manufacturer_GetAllManufacturers]    Script Date: 04/04/2013 11:42:28 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Manufacturer_GetById]
(
	@manufacturerId int --Manufacturer identifier
)
AS
BEGIN
SET NOCOUNT ON
select M.*
from Manufacturer M WITH (NOLOCK)
where M.Id=@manufacturerId
END
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Manufacturer_GetProductManufacturerById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Manufacturer_GetProductManufacturerById]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Manufacturer_GetProductManufacturerById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Manufacturer_GetProductManufacturerById]
GO
/****** Object:  StoredProcedure [FNS_Manufacturer_GetAllManufacturers]    Script Date: 04/04/2013 11:42:28 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Manufacturer_GetProductManufacturerById]
(
	@productManufacturerId int --Product manufacturer mapping identifier
)
AS
BEGIN
SET NOCOUNT ON
select M.*
from Product_Manufacturer_Mapping M WITH (NOLOCK)
where M.Id=@productManufacturerId
END
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Manufacturer_GetProductManufacturersByProductId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Manufacturer_GetProductManufacturersByProductId]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Manufacturer_GetProductManufacturersByProductId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Manufacturer_GetProductManufacturersByProductId]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Manufacturer_GetProductManufacturersByProductId]
(
	@productId int,	--Product identifie
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@StoreId int, -- Store mapping
	@ShowHidden			bit, --A value indicating whether to show hidden records
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit --A value indicating whether to Ignore Store Limitations
)
AS
BEGIN
SET NOCOUNT ON

if @ShowHidden=1
begin
	select Pm.*
	from Product_Manufacturer_Mapping Pm  WITH (NOLOCK),
		Manufacturer M WITH (NOLOCK)
	where Pm.ManufacturerId=M.Id and Pm.ProductId=@productId
		and M.Deleted=0
	Order by Pm.DisplayOrder, Pm.Id
end
else
begin
	--filter by customer role IDs (access control list)
	SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
	CREATE TABLE #FilteredCustomerRoleIds
	(
		CustomerRoleId int not null
	)
	INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

	select Pm.*
	from Product_Manufacturer_Mapping Pm  WITH (NOLOCK),
		Manufacturer M WITH (NOLOCK)
	where Pm.ManufacturerId=M.Id and Pm.ProductId=@productId
		and M.Deleted=0
		and M.Published =1
		and (@IgnoreAcl=1 or M.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl  WITH (NOLOCK)
									WHERE [acl].EntityId = M.Id AND [acl].EntityName = 'Manufacturer'
								)
							))
		and (@StoreId=0 or @IgnoreStoreLimitations=1 or M.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
			where sm.EntityName = 'Manufacturer' and sm.StoreId=@StoreId and sm.EntityId=M.Id))
	Order by Pm.DisplayOrder, Pm.Id
	
	drop table #FilteredCustomerRoleIds
end

END

GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Manufacturer_GetProductManufacturersByManufacturerId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Manufacturer_GetProductManufacturersByManufacturerId]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Manufacturer_GetProductManufacturersByManufacturerId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Manufacturer_GetProductManufacturersByManufacturerId]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO						
CREATE PROCEDURE [dbo].[FNS_Manufacturer_GetProductManufacturersByManufacturerId]
(
	@manufacturerId int,	--Manufacturer identifier
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@PageIndex			int , --Page index
	@PageSize			int, --Page size
	@StoreId			int, -- Store mapping
	@ShowHidden			bit , --A value indicating whether to show hidden records
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit, --A value indicating whether to Ignore Store Limitations
	@TotalRecords		int OUTPUT
)
AS
BEGIN
SET NOCOUNT ON

--paging
DECLARE @PageLowerBound int
DECLARE @PageUpperBound int
DECLARE @RowsToReturn int
SET @RowsToReturn = @PageSize * (@PageIndex + 1)	
SET @PageLowerBound = @PageSize * @PageIndex
SET @PageUpperBound = @PageLowerBound + @PageSize + 1

CREATE TABLE #DisplayOrderTmp 
(
	[Id] int IDENTITY (1, 1) NOT NULL,
	[ProductManufacturerId] int NOT NULL
)

if @ShowHidden=1
begin
	insert into #DisplayOrderTmp (ProductManufacturerId)
	select Pm.Id  
	from Product_Manufacturer_Mapping Pm  WITH (NOLOCK),
		Manufacturer M WITH (NOLOCK)
	where Pm.ManufacturerId=M.Id and Pm.ManufacturerId=@manufacturerId
		and M.Deleted=0
	Order by Pm.DisplayOrder, Pm.Id
end
else
begin
	--filter by customer role IDs (access control list)
	SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
	CREATE TABLE #FilteredCustomerRoleIds
	(
		CustomerRoleId int not null
	)
	INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

	insert into #DisplayOrderTmp (ProductManufacturerId)
	select Pm.Id  
	from Product_Manufacturer_Mapping Pm  WITH (NOLOCK),
		Manufacturer M WITH (NOLOCK)
	where Pm.ManufacturerId=M.Id and Pm.ManufacturerId=@manufacturerId
		and M.Deleted=0
		and M.Published =1
		and (@IgnoreAcl=1 or M.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl  WITH (NOLOCK)
									WHERE [acl].EntityId = M.Id AND [acl].EntityName = 'Manufacturer'
								)
							))
		and (@StoreId=0 or @IgnoreStoreLimitations=1 or M.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping SM  WITH (NOLOCK) 
			where SM.EntityName = 'Manufacturer' and SM.StoreId=@StoreId and SM.EntityId=M.Id))
	Order by Pm.DisplayOrder, Pm.Id

	drop table #FilteredCustomerRoleIds
end
					
	CREATE TABLE #PageIndex 
	(
		[IndexId] int IDENTITY (1, 1) NOT NULL,
		[ProductManufacturerId] int NOT NULL
	)
	INSERT INTO #PageIndex (ProductManufacturerId)
	SELECT ProductManufacturerId
	FROM #DisplayOrderTmp
	GROUP BY ProductManufacturerId
	ORDER BY min([Id])

	--total records
	SET @TotalRecords = @@rowcount
	
	DROP TABLE #DisplayOrderTmp

	--return categories
	SELECT TOP (@RowsToReturn)
		C.*
	FROM
		#PageIndex [pi]
		INNER JOIN Product_Manufacturer_Mapping C  WITH (NOLOCK) on C.Id = [pi].[ProductManufacturerId]
	WHERE
		[pi].IndexId > @PageLowerBound AND 
		[pi].IndexId < @PageUpperBound
	ORDER BY
		[pi].IndexId
	
	DROP TABLE #PageIndex
END
GO
