﻿IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Order_SearchOrders]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Order_SearchOrders]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Order_SearchOrders]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Order_SearchOrders]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Order_SearchOrders]
(
	@storeId int, -- Store identifier; 0 to load all orders
	@vendorId int, -- Vendor identifier; 0 to load all orders
	@customerId int, -- Customer identifier; 0 to load all orders
	@productId int, -- Product identifier which was purchased in an order; 0 to load all orders
	@affiliateId int, -- Affiliate identifier; 0 to load all orders
	@warehouseId int, --Warehouse identifier, only orders with products from a specified warehouse will be loaded; 0 to load all orders
	@billingCountryId int, --Billing country identifier; 0 to load all orders
	@paymentMethodSystemName nvarchar(4000) = null, --Payment method system name; null to load all records
	@createdFromUtc datetime = null, -- Created date from (UTC); null to load all records
	@createdToUtc datetime = null, -- Created date to (UTC); null to load all records
	@osIds nvarchar(MAX) = null,	--Order status identifiers; null to load all orders (comma-separated list)
	@psIds nvarchar(MAX) = null, -- Payment status identifiers; null to load all orders (comma-separated list)
	@ssIds nvarchar(MAX) = null, -- Shipping status identifiers; null to load all orders (comma-separated list)
	@billingPhone nvarchar(4000) = null, -- Billing phone. Leave empty to load all records.
	@billingEmail nvarchar(4000) = null, -- Billing email. Leave empty to load all records.
	@billingLastName nvarchar(4000) = null, -- Billing last name. Leave empty to load all records.
	@orderNotes nvarchar(4000) = null, --Search in order notes. Leave empty to load all records.
	@PageIndex			int = 0, --Page index
	@PageSize			int = 2147483644, --Page size
	@TotalRecords		int = null OUTPUT
)
AS
BEGIN
SET NOCOUNT ON

SET @osIds = isnull(@osIds, '')	
CREATE TABLE #tableosIds
(
	orderid int identity(1,1),
	Id int not null
)
INSERT INTO #tableosIds (Id)
SELECT CAST(data as int) FROM [nop_splitstring_to_table](@osIds, ',')	

SET @psIds = isnull(@psIds, '')	
CREATE TABLE #tablepsIds
(
	orderid int identity(1,1),
	Id int not null
)
INSERT INTO #tablepsIds (Id)
SELECT CAST(data as int) FROM [nop_splitstring_to_table](@psIds, ',')	

SET @ssIds = isnull(@ssIds, '')	
CREATE TABLE #tablessIds
(
	orderid int identity(1,1),
	Id int not null
)
INSERT INTO #tablessIds (Id)
SELECT CAST(data as int) FROM [nop_splitstring_to_table](@ssIds, ',')	
		

DECLARE @sql nvarchar(max)

--paging
DECLARE @PageLowerBound int
DECLARE @PageUpperBound int
DECLARE @RowsToReturn int
SET @RowsToReturn = @PageSize * (@PageIndex + 1)	
SET @PageLowerBound = @PageSize * @PageIndex
SET @PageUpperBound = @PageLowerBound + @PageSize + 1

CREATE TABLE #DisplayOrderTmp 
(
	[Id] int IDENTITY (1, 1) NOT NULL,
	[OrderId] int NOT NULL
)

--orders
SET @sql = '
	INSERT INTO #DisplayOrderTmp ([OrderId])
	SELECT O.Id
	FROM
	[Order] O with (NOLOCK)
	where O.Deleted=0 '

	IF @storeId>0
		SET @sql = @sql + ' and O.StoreId = ' + CAST(@storeId AS nvarchar(max))

	IF @customerId>0
		SET @sql = @sql + ' and O.CustomerId = ' + CAST(@customerId AS nvarchar(max))
    if (@affiliateId > 0)
		SET @sql = @sql + ' and O.AffiliateId = ' + CAST(@affiliateId AS nvarchar(max))

	if @vendorId > 0 or @productId > 0 or @warehouseId > 0
	begin
		SET @sql = @sql + ' and O.id in (select oi.OrderId from OrderItem oi WITH (NOLOCK) where oi.ProductId in (select P.Id from Product P  WITH (NOLOCK) where '

            if @vendorId > 0
				SET @sql = @sql + ' P.VendorId =' + CAST(@vendorId AS nvarchar(max))

            if @productId > 0
			begin
				if @vendorId > 0
				SET @sql = @sql + ' and '

				SET @sql = @sql + ' P.Id =' + CAST(@productId AS nvarchar(max))
			end
            if @warehouseId > 0
			begin
				if @productId > 0 or @vendorId > 0
				SET @sql = @sql + ' and '

/*				SET @sql = @sql + ' P.WarehouseId =' + CAST(@warehouseId AS nvarchar(max))*/
				SET @sql = @sql + ' ((P.ManageInventoryMethodId=1 and P.UseMultipleWarehouses=1 and P.WarehouseId =' + CAST(@warehouseId AS nvarchar(max))+') or '+
					' ((P.ManageInventoryMethodId!=1 or P.UseMultipleWarehouses!=1) and P.WarehouseId =' + CAST(@warehouseId AS nvarchar(max))+') )'
			end
		SET @sql = @sql + ' ))'
	end

    if @createdFromUtc is not null
		SET @sql = @sql + ' and O.CreatedOnUtc >= convert(datetime,'''+CONVERT(nvarchar(25),@createdFromUtc,120)+''',120)'

    if @createdToUtc is not null
		SET @sql = @sql + ' and O.CreatedOnUtc <= convert(datetime,'''+CONVERT(nvarchar(25),@createdToUtc,120)+''',120)'

    if @osIds!=''
		SET @sql = @sql + ' and O.OrderStatusId in (select Id from #tableosIds)'

    if @psIds!=''
		SET @sql = @sql + ' and O.PaymentStatusId in (select Id from #tablepsIds)'

    if @ssIds!=''
		SET @sql = @sql + ' and O.ShippingStatusId in (select Id from #tablessIds)'

    if ISNULL(@billingPhone,'')!='' or  ISNULL(@billingEmail,'')!='' or @billingCountryId>0 or ISNULL(@billingLastName,'')!='' 
	begin
		SET @sql = @sql + ' and O.BillingAddressId in (select Id from Address A WITH (NOLOCK) where '
		declare @addAND bit
		set @addAND=0
		if ISNULL(@billingPhone,'')!=''
		BEGIN
			SET @sql = @sql + 'A.PhoneNumber like ''%'+ltrim(rtrim(@billingPhone))+'%'''
			set @addAND=1
		END
		if ISNULL(@billingEmail,'')!=''
		BEGIN
			if @addAND=1
			BEGIN
				SET @sql = @sql + 'AND '
			END
			SET @sql = @sql + 'A.email like ''%'+ltrim(rtrim(@billingEmail))+'%'''
			set @addAND=1
		END
		if @billingCountryId>0
		BEGIN
			if @addAND=1
			BEGIN
				SET @sql = @sql + 'AND '
			END
			SET @sql = @sql + 'A.CountryId ='+CAST(@billingCountryId AS nvarchar(max))
			set @addAND=1
		END
		if ISNULL(@billingLastName,'')!=''
		BEGIN
			if @addAND=1
			BEGIN
				SET @sql = @sql + 'AND '
			END
			SET @sql = @sql + 'A.LastName like ''%'+ltrim(rtrim(@billingLastName))+'%'''
		END
		SET @sql = @sql + ' )'
	end

    if ISNULL(@paymentMethodSystemName,'')!=''
	begin
		SET @sql = @sql + ' and O.paymentMethodSystemName = '''+ltrim(rtrim(@paymentMethodSystemName))+''''
	end
	
    if ISNULL(@orderNotes,'')!=''
	begin
		SET @sql = @sql + ' and O.Id in (select OrderNote.OrderId from OrderNote WITH (NOLOCK) where OrderNote.Note like ''%'+ltrim(rtrim(@orderNotes))+'%'' )'
	end

	set @sql=@sql+ ' Order by O.CreatedOnUtc desc'

	--PRINT (@sql)
	EXEC sp_executesql @sql

	drop table #tableosIds
	drop table #tablepsIds
	drop table #tablessIds
					
	CREATE TABLE #PageIndex 
	(
		[IndexId] int IDENTITY (1, 1) NOT NULL,
		[OrderId] int NOT NULL
	)
	INSERT INTO #PageIndex ([OrderId])
	SELECT [OrderId]
	FROM #DisplayOrderTmp
	GROUP BY [OrderId]
	ORDER BY min([Id])

	--total records
	SET @TotalRecords = @@rowcount
	
	DROP TABLE #DisplayOrderTmp

	--return orders
	SELECT TOP (@RowsToReturn)
		O.*
	FROM
		#PageIndex [pi]
		INNER JOIN [Order] O WITH (NOLOCK) on O.Id = [pi].[OrderId]
	WHERE
		[pi].IndexId > @PageLowerBound AND 
		[pi].IndexId < @PageUpperBound
	ORDER BY
		[pi].IndexId
	
	DROP TABLE #PageIndex
END
GO
