﻿IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_CatalogModelFactory_PrepareCategorySimpleModels]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_CatalogModelFactory_PrepareCategorySimpleModels]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_CatalogModelFactory_PrepareCategorySimpleModels]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_CatalogModelFactory_PrepareCategorySimpleModels]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_CatalogModelFactory_PrepareCategorySimpleModels]
(
	@rootCategoryId int, --Root category identifier
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@StoreId int,	--storeId
	@LanguageId int=0, --LanguageId
	@calccount	bit =1, --A value indicating whether to calculate Products in categories
	@calccountsub	bit = 1, --A value indicating whether to calculate Products in subcategories,
	@IgnoreAcl			bit = 1,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit = 1, --A value indicating whether to Ignore Store Limitations
	@LoadSubCategories bit = 1 --A value indicating whether subcategories should be loaded
)
AS
BEGIN
SET NOCOUNT ON
	--filter by customer role IDs (access control list)
	SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
	CREATE TABLE #FilteredCustomerRoleIds
	(
		CustomerRoleId int not null
	)
	INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	
	
	create table #tmpCategory (Id int,Name nvarchar(400),ParentCategoryId int,
		DisplayOrder int,NumberOfProducts int,SeName nvarchar(400),IncludeInTopMenu bit, Levelsort int)

if @LoadSubCategories=1
begin
		--Category
		;with cte_tree (Id,ParentCategoryId,SubjectToAcl,LimitedToStores, Levelsort) as 
		(
			select C.Id,C.ParentCategoryId,C.SubjectToAcl,C.LimitedToStores, 0 as Levelsort
			from Category C WITH (NOLOCK)
			where C.ParentCategoryId=@rootCategoryId
				and C.Deleted=0
				and C.Published=1
			union all
			select T.Id,T.ParentCategoryId,T.SubjectToAcl,T.LimitedToStores, C.Levelsort+1
			from cte_tree C,Category T WITH (NOLOCK)
			where C.Id=T.ParentCategoryId
				and T.Deleted=0
				and T.Published=1
		)

		insert into #tmpCategory (Id,Name,ParentCategoryId,DisplayOrder,NumberOfProducts,SeName,IncludeInTopMenu, Levelsort)
		select C.Id,C.Name,C.ParentCategoryId,C.DisplayOrder,0,'',C.IncludeInTopMenu, T.Levelsort
		from Category C WITH (NOLOCK), cte_tree T
		where C.Id = T.Id
			and (@IgnoreAcl=1 or C.SubjectToAcl = 0 OR EXISTS (
								SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
									WHERE
										[fcr].CustomerRoleId IN (
											SELECT [acl].CustomerRoleId
											FROM [AclRecord] acl  WITH (NOLOCK)
											WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
										)
									))
			and	(@IgnoreStoreLimitations=1 or @StoreId=0 or C.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
					where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))
								
	--remove categories without any parent categories
	while (1=1)
	begin
		delete #tmpCategory
		from #tmpCategory C
		where C.ParentCategoryId!=0 and C.ParentCategoryId not in (select Id from #tmpCategory)
		if @@ROWCOUNT=0
			break
	end
end
else
begin
		insert into #tmpCategory (Id,Name,ParentCategoryId,DisplayOrder,NumberOfProducts,SeName,IncludeInTopMenu, Levelsort)
		select C.Id,C.Name,C.ParentCategoryId,C.DisplayOrder,0,'',C.IncludeInTopMenu, 0 as Levelsort
		from Category C WITH (NOLOCK)
		where C.ParentCategoryId=@rootCategoryId
				and C.Deleted=0
				and C.Published=1
			and (@IgnoreAcl=1 or C.SubjectToAcl = 0 OR EXISTS (
								SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
									WHERE
										[fcr].CustomerRoleId IN (
											SELECT [acl].CustomerRoleId
											FROM [AclRecord] acl  WITH (NOLOCK)
											WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
										)
									))
			and	(@IgnoreStoreLimitations=1 or @StoreId=0 or C.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
					where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))
end
--calculate products
if @calccount=1
begin
	create table #ProductCategoryMapping (CategoryId int, ProductId int)
	insert into #ProductCategoryMapping (CategoryId, ProductId)			
	SELECT PC.CategoryId, PC.ProductId
	FROM Product_Category_Mapping PC WITH (NOLOCK),Product P WITH (NOLOCK)
	WHERE PC.CategoryId in (select Id from #tmpCategory)
		and PC.ProductId=P.Id
		and P.Published=1 and P.Deleted=0
		AND P.VisibleIndividually = 1
				and (@IgnoreAcl=1 or P.SubjectToAcl = 0 OR EXISTS (
										SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
											WHERE
												[fcr].CustomerRoleId IN (
													SELECT [acl].CustomerRoleId
													FROM [AclRecord] acl  WITH (NOLOCK)
													WHERE [acl].EntityId = P.Id AND [acl].EntityName = 'Product'
												)
											))
				and (@IgnoreStoreLimitations=1 or @StoreId=0 or P.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm WITH (NOLOCK)
					where sm.EntityName = 'Product' and sm.StoreId=@StoreId and sm.EntityId=P.Id))

	if @calccountsub=1
	begin
		while (1=1)
		begin
			insert into #ProductCategoryMapping (CategoryId, ProductId)	
			select C.ParentCategoryId, PM.ProductId
			from #ProductCategoryMapping PM, #tmpCategory C
			where PM.CategoryId=C.Id 
				and not exists(select PM2.ProductId from #ProductCategoryMapping PM2 where PM2.CategoryId=C.ParentCategoryId and PM2.ProductId=PM.ProductId)

			if @@ROWCOUNT=0
				break
		end
	end

	update C
	set NumberOfProducts=R.NumberOfProducts
	from #tmpCategory C,
		(select PM.CategoryId,COUNT(distinct ProductId) as NumberOfProducts
		from #ProductCategoryMapping PM
		group by PM.CategoryId
		) as R
		where C.Id=R.CategoryId

	drop table #ProductCategoryMapping
end

drop table #FilteredCustomerRoleIds

update C
set SeName=ltrim(rtrim(Substring(US.Slug,1,400)))
from #tmpCategory C, (select TOP 1 WITH TIES U.EntityId,U.Slug
	from UrlRecord U WITH (NOLOCK)
	where U.EntityId in (select Id from #tmpCategory) 
		and U.EntityName='Category' and U.LanguageId=0 and U.IsActive=1
	ORDER BY ROW_NUMBER() OVER(PARTITION BY U.EntityId ORDER BY U.Id DESC)) US
where C.Id=US.EntityId
		
if @LanguageId>0
begin
	update G
	set Name=ISNULL(L.LocaleValue,G.name)
	from #tmpCategory G left join LocalizedProperty L on G.Id=L.EntityId 
		and L.LocaleKeyGroup='Category' 
		and L.LocaleKey='Name' 
		and L.LanguageId=@LanguageId
		
	update C
	set SeName=ltrim(rtrim(Substring(US.Slug,1,400)))
	from #tmpCategory C, (select TOP 1 WITH TIES U.EntityId,U.Slug
		from UrlRecord U WITH (NOLOCK)
		where U.EntityId in (select Id from #tmpCategory) 
			and U.EntityName='Category' and U.LanguageId=@LanguageId and U.IsActive=1
			and ltrim(rtrim(U.Slug))!=''
		ORDER BY ROW_NUMBER() OVER(PARTITION BY U.EntityId ORDER BY U.Id DESC)) US
	where C.Id=US.EntityId
end			

select C.Id,C.Name,C.ParentCategoryId,C.SeName,C.NumberOfProducts,C.IncludeInTopMenu,C.DisplayOrder
from #tmpCategory C
order by C.ParentCategoryId, C.DisplayOrder, C.Id

drop table #tmpCategory
end

GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_CatalogModelFactory_PrepareTopMenuTopicModel]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_CatalogModelFactory_PrepareTopMenuTopicModel]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_CatalogModelFactory_PrepareTopMenuTopicModel]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_CatalogModelFactory_PrepareTopMenuTopicModel]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_CatalogModelFactory_PrepareTopMenuTopicModel]
(
	@StoreId int,	--storeId
	@LanguageId int=0 --LanguageId
)
AS
BEGIN
SET NOCOUNT ON

create table #tmpTopic (Id int, SystemName nvarchar(max), Title nvarchar(max), SeName nvarchar(400))
insert into #tmpTopic (Id, SystemName, Title, SeName)
select T.Id, T.SystemName,T.Title,''
from Topic T WITH (NOLOCK)
where T.IncludeInTopMenu=1
	and (@StoreId=0 or T.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping SM   WITH (NOLOCK)
			where SM.EntityName = 'Topic' and SM.StoreId=@StoreId and SM.EntityId=T.Id))

update T
set SeName=ltrim(rtrim(Substring(US.Slug,1,400)))
from #tmpTopic T, (select TOP 1 WITH TIES U.EntityId,U.Slug
	from UrlRecord U WITH (NOLOCK)
	where U.EntityId in (select Id from #tmpTopic) 
		and U.EntityName='Topic' and U.LanguageId=0 and U.IsActive=1
	ORDER BY ROW_NUMBER() OVER(PARTITION BY U.EntityId ORDER BY U.Id DESC)) US
where T.Id=US.EntityId
		
if @LanguageId>0
begin
	update T
	set Title=ISNULL(L.LocaleValue,T.Title)
	from #tmpTopic T left join LocalizedProperty L WITH (NOLOCK) on T.Id=L.EntityId 
		and L.LocaleKeyGroup='Topic' 
		and L.LocaleKey='Title' 
		and L.LanguageId=@LanguageId
		
	update T
	set SeName=ltrim(rtrim(Substring(US.Slug,1,400)))
	from #tmpTopic T, (select TOP 1 WITH TIES U.EntityId,U.Slug
		from UrlRecord U WITH (NOLOCK)
		where U.EntityId in (select Id from #tmpTopic) 
			and U.EntityName='Topic' and U.LanguageId=@LanguageId and U.IsActive=1
			and ltrim(rtrim(U.Slug))!=''
		ORDER BY ROW_NUMBER() OVER(PARTITION BY U.EntityId ORDER BY U.Id DESC)) US
	where T.Id=US.EntityId
end			

select T.Id, T.Title as Name, T.SeName
from #tmpTopic T
order by T.SystemName

drop table #tmpTopic
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_CatalogModelFactory_HomepageCategories]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_CatalogModelFactory_HomepageCategories]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_CatalogModelFactory_HomepageCategories]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_CatalogModelFactory_HomepageCategories]
GO
CREATE PROCEDURE [dbo].[FNS_CatalogModelFactory_HomepageCategories]
(
	@StoreId			int = 0,
	@LanguageId			int = 0,
	@AllowedCustomerRoleIds	nvarchar(MAX) = null,	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit --A value indicating whether to Ignore Store Limitations
)
AS
SET NOCOUNT ON
BEGIN

--filter by customer role IDs (access control list)
SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
CREATE TABLE #FilteredCustomerRoleIds
(
	CustomerRoleId int not null
)
INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	
	
if @LanguageId>0
begin	
	declare @countLanguage int
	set @countLanguage=0
	select @countLanguage=count(*)
	from Language L WITH (NOLOCK)
	/*where L.Published=1
			and (@StoreId=0 or L.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
				where sm.EntityName = 'Language' and sm.StoreId=@StoreId and sm.EntityId=L.Id))*/
	if @countLanguage=1
	begin
		set @LanguageId=0
	end
end
	create table #tmpCategory (Id int,[Name] nvarchar(400),[Description] nvarchar(max),MetaKeywords nvarchar(400),
		MetaDescription nvarchar(max),MetaTitle nvarchar(400),PictureId int,DisplayOrder int, SeName nvarchar(400))

	insert into #tmpCategory (Id,Name,[Description],MetaKeywords,MetaDescription,MetaTitle,PictureId,DisplayOrder,SeName)
	select C.Id,C.Name,C.[Description],C.MetaKeywords,C.MetaDescription,C.MetaTitle,C.PictureId,C.DisplayOrder,'' as SeName
	from Category C  WITH (NOLOCK)
	where C.ShowOnHomePage=1
		and C.Deleted=0
		and C.Published=1
		and (@IgnoreAcl=1 or C.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl  WITH (NOLOCK)
									WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
								)
							))
		and (@StoreId=0 or @IgnoreStoreLimitations=1 or C.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
			where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))
	Order by C.DisplayOrder		

update C
set SeName=ltrim(rtrim(Substring(US.Slug,1,400)))
from #tmpCategory C, (select TOP 1 WITH TIES U.EntityId,U.Slug
	from UrlRecord U WITH (NOLOCK)
	where U.EntityId in (select Id from #tmpCategory) 
		and U.EntityName='Category' and U.LanguageId=0 and U.IsActive=1
	ORDER BY ROW_NUMBER() OVER(PARTITION BY U.EntityId ORDER BY U.Id DESC)) US
where C.Id=US.EntityId
		
if @LanguageId>0
begin
	create table #tmpLocalizedProperty (EntityId int, LocaleKey nvarchar(400), LocaleValue nvarchar(max))
	insert into #tmpLocalizedProperty (EntityId, LocaleKey, LocaleValue)
	select L.EntityId, L.LocaleKey, L.LocaleValue
	from LocalizedProperty L
	where L.LocaleKeyGroup='Category'
		and L.LanguageId=@LanguageId
		and L.EntityId in (select Id from #tmpCategory)

	update G
	set Name=ISNULL(L.LocaleValue,G.name)
	from #tmpCategory G left join #tmpLocalizedProperty L on G.Id=L.EntityId 
		and L.LocaleKey='Name' 

	update G
	set [Description]=ISNULL(L.LocaleValue,G.[Description])
	from #tmpCategory G left join #tmpLocalizedProperty L on G.Id=L.EntityId 
		and L.LocaleKey='Description' 

	update G
	set MetaKeywords=ISNULL(L.LocaleValue,G.MetaKeywords)
	from #tmpCategory G left join #tmpLocalizedProperty L on G.Id=L.EntityId 
		and L.LocaleKey='MetaKeywords' 

	update G
	set MetaDescription=ISNULL(L.LocaleValue,G.MetaDescription)
	from #tmpCategory G left join #tmpLocalizedProperty L on G.Id=L.EntityId 
		and L.LocaleKey='MetaDescription' 

	update G
	set MetaTitle=ISNULL(L.LocaleValue,G.MetaTitle)
	from #tmpCategory G left join #tmpLocalizedProperty L on G.Id=L.EntityId 
		and L.LocaleKey='MetaTitle' 
	
	drop table #tmpLocalizedProperty
								
	update C
	set SeName=ltrim(rtrim(Substring(US.Slug,1,400)))
	from #tmpCategory C, (select TOP 1 WITH TIES U.EntityId,U.Slug
		from UrlRecord U WITH (NOLOCK)
		where U.EntityId in (select Id from #tmpCategory) 
			and U.EntityName='Category' and U.LanguageId=@LanguageId and U.IsActive=1
			and ltrim(rtrim(U.Slug))!=''
		ORDER BY ROW_NUMBER() OVER(PARTITION BY U.EntityId ORDER BY U.Id DESC)) US
	where C.Id=US.EntityId
end			

	select Id,Name,[Description],MetaKeywords,MetaDescription,MetaTitle,PictureId,SeName 
	from #tmpCategory 
	Order by DisplayOrder
END
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_CatalogModelFactory_ProductLoadAllPaged_GetMinimumPrice]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_CatalogModelFactory_ProductLoadAllPaged_GetMinimumPrice]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_CatalogModelFactory_ProductLoadAllPaged_GetMinimumPrice]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_CatalogModelFactory_ProductLoadAllPaged_GetMinimumPrice]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_CatalogModelFactory_ProductLoadAllPaged_GetMinimumPrice]
(
	@StoreId			int = 0,
	@grouped bit =0
)
AS
BEGIN
SET NOCOUNT ON
--#ProductId (Id, ParentGroupedProductId, Price, HasTierPrices)

--TierPrice
create table #ProductsTierPrice (id int not null)
insert into #ProductsTierPrice (id)
select distinct Id 
from #ProductId P 
where P.HasTierPrices=1
			
if exists(select * from #ProductsTierPrice)
begin
	create table #TierPrice (ProductId int not null, Price decimal(18, 4) NOT NULL,Quantity int not null)
	insert into #TierPrice (ProductId,Price,Quantity)
	select T.ProductId,T.Price,T.Quantity
	from TierPrice T WITH (NOLOCK)
	where (T.StoreId=0 or T.StoreId=@StoreId) 
		and (T.CustomerRoleId is null or T.CustomerRoleId in (select CR.CustomerRoleId from #FilteredCustomerRoleIds CR))
		and T.ProductId in (select Id from #ProductsTierPrice)
		AND (getutcdate() BETWEEN ISNULL(T.StartDateTimeUtc, '1/1/1900') and ISNULL(T.EndDateTimeUtc, '1/1/2999'))

	update #ProductId
	set Price=case
		when P.Price>TR.Price then TR.Price
		else P.Price
		end
	from #ProductId P,(select T.ProductId,min(T.Price) as Price
		from #TierPrice T 
			group by T.ProductId
				) as TR
	where P.Id=TR.ProductId
				
	drop table #TierPrice
end
drop table #ProductsTierPrice
--end TierPrice

--grouped product, get only one
if @grouped=1
begin
	delete from #ProductId
	where Id not in (
				SELECT TOP 1 WITH TIES P.Id
				FROM #ProductId P WITH (NOLOCK)
				ORDER BY ROW_NUMBER() OVER(PARTITION BY P.ParentGroupedProductId ORDER BY P.Price ASC))
end
--end TierPrice
END

GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_CatalogModelFactory_ProductLoadAllPaged]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_CatalogModelFactory_ProductLoadAllPaged]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_CatalogModelFactory_ProductLoadAllPaged]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_CatalogModelFactory_ProductLoadAllPaged]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_CatalogModelFactory_ProductLoadAllPaged]
(
	@CategoryId		int = 0,	
	@ManufacturerId		int = 0,
	@VendorId			int = 0,
	@StoreId			int = 0,
	@LanguageId			int = 0,
	@AllowedCustomerRoleIds	nvarchar(MAX) = null,	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@getCategorybreadcrumb bit=1, --Get Category breadcrumb
	@getSubCategories bit=1, --Get Sub Categories for selected category
	@getFeaturedProducts bit=1, --Get Featured Products for selected category/Manufacturer/Vendor
	@getProducts bit=1, --Get Products for selected category/Manufacturer/Vendor pages
	@ShowProductsFromSubcategories bit=1, --Get Products From Subcategories
	@IgnoreDiscounts bit =1, --Ignore Discounts 
	@PrepareSpecificationAttributes bit =1, --Prepare Specification Attributes
	@PageIndex			int = 0, 
	@PageSize			int = 2147483644,
	@FeaturedProducts	bit = null,	--0 featured only , 1 not featured only, null - load all products
	@PriceMin			decimal(18, 4) = null,
	@PriceMax			decimal(18, 4) = null,
	@OrderBy			int = 0, --0 - position, 5 - Name: A to Z, 6 - Name: Z to A, 10 - Price: Low to High, 11 - Price: High to Low, 15 - creation date
	@FilteredSpecs		nvarchar(MAX) = null,	--filter by attributes (comma-separated list). e.g. 14,15,16
	@LoadFilterableSpecificationAttributeOptionIds bit = 0, --a value indicating whether we should load the specification attribute option identifiers applied to loaded products (all pages)
	@FilterableSpecificationAttributeOptionIds nvarchar(MAX) = null, --the specification attribute option identifiers applied to loaded products (all pages). returned as a comma separated list of identifiers
	@TotalRecords		int = null
)
AS
SET NOCOUNT ON
BEGIN

--filter by customer role IDs (access control list)
SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
CREATE TABLE #FilteredCustomerRoleIds
(
	CustomerRoleId int not null
)
INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	
	
if @LanguageId>0
begin	
	declare @countLanguage int
	set @countLanguage=0
	select @countLanguage=count(*)
	from Language L WITH (NOLOCK)
	/*where L.Published=1
			and (@StoreId=0 or L.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
				where sm.EntityName = 'Language' and sm.StoreId=@StoreId and sm.EntityId=L.Id))*/
	if @countLanguage=1
	begin
		set @LanguageId=0
	end
end

create table #UrlRecord	(ProductId int,SeName nvarchar(400))
create table #TierPriceId (ProductId int)
create table #ProductId (Id int, ParentGroupedProductId int, Price decimal(18,4), HasTierPrices bit )
create table #ProductSpecificationAttributeMapping (Id int)

--category breadcrumb
if @getCategorybreadcrumb=1
begin
	create table #CategoryBreadCrumb (Id int,Name nvarchar(400),SeName nvarchar(400),Levelsort int)
	if (@CategoryId>0)
	begin
		--Category BreadCrumb tree
		;with cte_tree_BreadCrumb (Id,Name,ParentCategoryId,Levelsort) as 
		(
			select C.Id,C.Name,C.ParentCategoryId,0 as Levelsort
			from Category C
			where C.Id=@CategoryId and C.Deleted=0 and C.Published=1
					and (C.SubjectToAcl = 0 OR EXISTS (
							SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
								WHERE
									[fcr].CustomerRoleId IN (
										SELECT [acl].CustomerRoleId
										FROM [AclRecord] acl
										WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
									)
								))
					and (@StoreId=0 or C.LimitedToStores=0 or Exists(select Id from StoreMapping sm   WITH (NOLOCK)
						where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))
			union all
			select C.Id,C.Name,C.ParentCategoryId,T.Levelsort+1 as Levelsort
			from cte_tree_BreadCrumb T,Category C WITH (NOLOCK)
			where T.ParentCategoryId=C.Id and C.Deleted=0 and C.Published=1
					and (C.SubjectToAcl = 0 OR EXISTS (
							SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
								WHERE
									[fcr].CustomerRoleId IN (
										SELECT [acl].CustomerRoleId
										FROM [AclRecord] acl
										WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
									)
								))
					and (@StoreId=0 or C.LimitedToStores=0 or Exists(select Id from StoreMapping sm   WITH (NOLOCK)
						where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))
		)
		
		insert  into #CategoryBreadCrumb (Id,Name,SeName,Levelsort)
		select Id,Name,'',Levelsort
		from cte_tree_BreadCrumb C

		if exists(select * from #CategoryBreadCrumb)
		begin
			update #CategoryBreadCrumb
			set SeName=ltrim(rtrim(Substring(U.Slug,1,400)))
			from #CategoryBreadCrumb C, UrlRecord U WITH (NOLOCK)
			where C.Id=U.EntityId and U.EntityName='Category' and U.LanguageId=0 and U.IsActive=1	
			
			if (@LanguageId>0)
			begin
				update #CategoryBreadCrumb
				set Name=ltrim(rtrim(Substring(L.LocaleValue,1,400)))
				from #CategoryBreadCrumb C, 
					LocalizedProperty L WITH (NOLOCK) 
				where C.Id=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='Category' and L.LocaleKey='Name'
			
				update #CategoryBreadCrumb
				set SeName=ltrim(rtrim(Substring(U.Slug,1,400)))
				from #CategoryBreadCrumb C, UrlRecord U WITH (NOLOCK)
				where C.Id=U.EntityId and U.EntityName='Category' and U.LanguageId=@LanguageId and U.IsActive=1
			end
		end
	end
	 
	select Id,Name,SeName
	from #CategoryBreadCrumb
	order by Levelsort Desc
	drop table #CategoryBreadCrumb 
end
--End category breadcrumb

--SubCategories
if @getSubCategories=1
begin
	create table #SubCategoryIdBreadCrumb (Id int,Name nvarchar(400),[Description] nvarchar(max), SeName nvarchar(400),DisplayOrder int, PictureId int)
	if (@CategoryId>0)
	begin
		insert into #SubCategoryIdBreadCrumb (Id,Name,[Description],SeName,DisplayOrder,PictureId)
		select C.Id,C.Name,C.[Description],'',C.DisplayOrder,C.PictureId
		from Category C WITH (NOLOCK)
		where C.ParentCategoryId=@CategoryId and C.Deleted=0 and C.Published=1
				and (C.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl
									WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
								)
							))
				and (@StoreId=0 or C.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
					where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))


		if exists(select * from #SubCategoryIdBreadCrumb)
		begin
			update #SubCategoryIdBreadCrumb
			set SeName=ltrim(rtrim(Substring(U.Slug,1,400)))
			from #SubCategoryIdBreadCrumb C, UrlRecord U WITH (NOLOCK)
			where C.Id=U.EntityId and U.EntityName='Category' and U.LanguageId=0 and U.IsActive=1	
		
			if (@LanguageId>0)
			begin		
				update #SubCategoryIdBreadCrumb
				set Name=ltrim(rtrim(Substring(L.LocaleValue,1,400)))
				from #SubCategoryIdBreadCrumb C, 
					LocalizedProperty L WITH (NOLOCK) 
				where C.Id=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='Category' and L.LocaleKey='Name'
		
				update #SubCategoryIdBreadCrumb
				set [Description]=ltrim(rtrim(L.LocaleValue))
				from #SubCategoryIdBreadCrumb C, 
					LocalizedProperty L WITH (NOLOCK) 
				where C.Id=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='Category' and L.LocaleKey='Description'
		
				update #SubCategoryIdBreadCrumb
				set SeName=ltrim(rtrim(Substring(U.Slug,1,400)))
				from #SubCategoryIdBreadCrumb C, UrlRecord U WITH (NOLOCK)
				where C.Id=U.EntityId and U.EntityName='Category' and U.LanguageId=@LanguageId and U.IsActive=1
			end	
		end

	end

	select Id,Name,SeName,[Description], PictureId
	from #SubCategoryIdBreadCrumb
	order by DisplayOrder
	drop table #SubCategoryIdBreadCrumb 
end
--End SubCategories

--FeaturedProducts
if @getFeaturedProducts=1
begin
	create table #FeaturedProductsIds (Id int,DisplayOrder int)
	IF @CategoryId > 0
	BEGIN
		insert into #FeaturedProductsIds (Id,DisplayOrder)
		select distinct pcm.ProductId,pcm.DisplayOrder
		from Product_Category_Mapping pcm with (NOLOCK)
		where pcm.CategoryId=@CategoryId and pcm.IsFeaturedProduct=1
	END
	
	IF @ManufacturerId > 0
	BEGIN
		insert into #FeaturedProductsIds (Id,DisplayOrder)
		select distinct pmm.ProductId,pmm.DisplayOrder
		from Product_Manufacturer_Mapping pmm with (NOLOCK)
		where pmm.ManufacturerId=@CategoryId and pmm.IsFeaturedProduct=1
	END
	--return products
					
	create table #FeaturedProducts (Id int Not null, ProductTypeId int,ParentGroupedProductId int, Name nvarchar(400), ShortDescription nvarchar(max), FullDescription nvarchar(max),
		Price decimal(18, 4) NOT NULL, HasTierPrices bit NOT NULL, IndexId int not null) 

	insert into #FeaturedProducts (Id,ProductTypeId,ParentGroupedProductId,Name,ShortDescription,FullDescription,
		Price,HasTierPrices, IndexId)
	select P.Id,P.ProductTypeId,P.ParentGroupedProductId,P.Name,P.ShortDescription,P.FullDescription,
		P.Price,P.HasTierPrices, F.DisplayOrder
	from Product P With (NOLOCK),#FeaturedProductsIds F
	where P.Id=F.Id and P.Published=1 and P.Deleted=0
		and P.VisibleIndividually=1 
				and (P.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl
									WHERE [acl].EntityId = P.Id AND [acl].EntityName = 'Product'
								)
							))
				and (@StoreId=0 or P.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
					where sm.EntityName = 'Product' and sm.StoreId=@StoreId and sm.EntityId=P.Id))
	--order by F.DisplayOrder

	drop table #FeaturedProductsIds

	if exists(select * from #FeaturedProducts)
	begin
			if (@LanguageId>0)
			begin
				update #FeaturedProducts
				set Name=ltrim(rtrim(Substring(L.LocaleValue,1,400)))
				from #FeaturedProducts C, 
					LocalizedProperty L WITH (NOLOCK) 
				where C.Id=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='Product' and L.LocaleKey='Name'
			
				update #FeaturedProducts
				set ShortDescription=L.LocaleValue
				from #FeaturedProducts C, 
					LocalizedProperty L WITH (NOLOCK) 
				where C.Id=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='Product' and L.LocaleKey='ShortDescription'

				update #FeaturedProducts
				set FullDescription=L.LocaleValue
				from #FeaturedProducts C, 
					LocalizedProperty L WITH (NOLOCK) 
				where C.Id=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='Product' and L.LocaleKey='FullDescription'
			end
			
			if @IgnoreDiscounts=1
			begin
				delete from #ProductId
				insert into #ProductId (Id, ParentGroupedProductId, Price, HasTierPrices)
				select Id,ParentGroupedProductId,
					Price, HasTierPrices
				from #FeaturedProducts
				where ProductTypeId=5
				
				exec [FNS_CatalogModelFactory_ProductLoadAllPaged_GetMinimumPrice] @StoreId, 0
				
				update #FeaturedProducts
				set Price=P.Price
				from #FeaturedProducts F, #ProductId P
				where F.Id=P.Id
			end		
		
			--TierPrice
			insert into #TierPriceId (ProductId)
			select Id
			from #FeaturedProducts
			where HasTierPrices=1
			
			--UrlRecord SeName
			insert into #UrlRecord (ProductId,SeName)
			select TOP 1 WITH TIES U.EntityId as ProductId,ltrim(rtrim(Substring(U.Slug,1,400))) as SeName
			from UrlRecord U WITH (NOLOCK)
			where U.EntityId in (select Id from #FeaturedProducts) 
				and U.EntityName='Product' and U.LanguageId=0 and U.IsActive=1	
			ORDER BY ROW_NUMBER() OVER(PARTITION BY U.EntityId ORDER BY U.Id DESC)

			--Product Specification Attribute
			if @PrepareSpecificationAttributes=1
			begin
				insert into #ProductSpecificationAttributeMapping(Id)
				select Id 
				from Product_SpecificationAttribute_Mapping psm WITH (NOLOCK)
				where psm.ProductId in (select Id from #FeaturedProducts) 
			end
		end
		
	--Featured Products	
	select F.Price, F.[Name], F.ShortDescription, F.FullDescription, P.* 
	from Product P WITH(NOLOCK),#FeaturedProducts F
	where P.Id=F.Id
	order by F.IndexId

	--Group Product
	create table #FeaturedGroupProducts (Id int Not null, ProductTypeId int,ParentGroupedProductId int, 
		Price decimal(18, 4) NOT NULL, HasTierPrices bit NOT NULL) 

	insert into #FeaturedGroupProducts (Id,ProductTypeId,ParentGroupedProductId,
		Price,HasTierPrices)
	select P.Id,P.ProductTypeId,P.ParentGroupedProductId,
		P.Price,P.HasTierPrices
	from Product P With (NOLOCK)
	where P.ParentGroupedProductId in (select Id from #FeaturedProducts PG where PG.ProductTypeId=10)
		and P.Published=1 and P.Deleted=0
		and (P.SubjectToAcl = 0 OR EXISTS (
							SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
								WHERE
									[fcr].CustomerRoleId IN (
										SELECT [acl].CustomerRoleId
										FROM [AclRecord] acl
										WHERE [acl].EntityId = P.Id AND [acl].EntityName = 'Product'
									)
								))
		and (@StoreId=0 or P.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm  WITH (NOLOCK)
				where sm.EntityName = 'Product' and sm.StoreId=@StoreId and sm.EntityId=P.Id))
	
	if @IgnoreDiscounts=1
	begin
		delete from #ProductId
		
		insert into #ProductId (Id, ParentGroupedProductId, Price, HasTierPrices)
		select P.Id, P.ParentGroupedProductId, P.Price, P.HasTierPrices
		from #FeaturedGroupProducts P
		where P.ProductTypeId=5
					
		exec [FNS_CatalogModelFactory_ProductLoadAllPaged_GetMinimumPrice] @StoreId, 1
			
		delete from #FeaturedGroupProducts
		where ProductTypeId=5 and Id not in (select Id from #ProductId)	
		
		update #FeaturedGroupProducts
		set Price=P.Price
		from #FeaturedGroupProducts F, #ProductId P
		where F.Id=P.Id
		
		delete from #ProductId
	end		

	select F.Price, P.* 
	from Product P WITH(NOLOCK),#FeaturedGroupProducts F
	where P.Id=F.Id

	insert into #TierPriceId (ProductId)
	select Id
	from #FeaturedGroupProducts
	where HasTierPrices=1

	--Product Specification Attribute
	if @PrepareSpecificationAttributes=1
	begin
		insert into #ProductSpecificationAttributeMapping(Id)
		select Id 
		from Product_SpecificationAttribute_Mapping psm WITH (NOLOCK)
		where psm.ProductId in (select Id from #FeaturedGroupProducts) 
	end
				
	drop table #FeaturedProducts
	drop table #FeaturedGroupProducts
end
--End FeaturedProducts

--Products
if @getProducts=1
begin

	DECLARE
		@sql nvarchar(max),
		@sql_orderby nvarchar(max)

	--filter by category IDs
	CREATE TABLE #FilteredCategoryIds
	(
		CategoryId int not null
	)

	if (@CategoryId>0)
	begin
		if @ShowProductsFromSubcategories=1
		begin
		--Category Child tree
		;with cte_tree_CategoryChild (Id,ParentCategoryId) as 
		(
			select C.Id,C.ParentCategoryId
			from Category C
			where C.Id=@CategoryId and C.Deleted=0 and C.Published=1
					and (C.SubjectToAcl = 0 OR EXISTS (
							SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
								WHERE
									[fcr].CustomerRoleId IN (
										SELECT [acl].CustomerRoleId
										FROM [AclRecord] acl
										WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
									)
								))
					and (@StoreId=0 or C.LimitedToStores=0 or Exists(select Id from StoreMapping sm   WITH (NOLOCK)
						where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))
			union all
			select C.Id,C.ParentCategoryId
			from cte_tree_CategoryChild T,Category C WITH (NOLOCK)
			where T.Id=C.ParentCategoryId and C.Deleted=0 and C.Published=1
					and (C.SubjectToAcl = 0 OR EXISTS (
							SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
								WHERE
									[fcr].CustomerRoleId IN (
										SELECT [acl].CustomerRoleId
										FROM [AclRecord] acl
										WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
									)
								))
					and (@StoreId=0 or C.LimitedToStores=0 or Exists(select Id from StoreMapping sm   WITH (NOLOCK)
						where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))
		)
		insert  into #FilteredCategoryIds (CategoryId)
		select Id
		from cte_tree_CategoryChild C
		end
		else
		begin
			insert  into #FilteredCategoryIds (CategoryId)
			values (@CategoryId)
		end
	end
		
	DECLARE @CategoryIdsCount int	
	SET @CategoryIdsCount = (SELECT COUNT(1) FROM #FilteredCategoryIds)

	--filter by attributes
	SET @FilteredSpecs = isnull(@FilteredSpecs, '')	
	CREATE TABLE #FilteredSpecs
	(
		SpecificationAttributeOptionId int not null
	)
	INSERT INTO #FilteredSpecs (SpecificationAttributeOptionId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@FilteredSpecs, ',')
	DECLARE @SpecAttributesCount int	
	SET @SpecAttributesCount = (SELECT COUNT(1) FROM #FilteredSpecs)

	
	--paging
	DECLARE @PageLowerBound int
	DECLARE @PageUpperBound int
	DECLARE @RowsToReturn int
	SET @RowsToReturn = @PageSize * (@PageIndex + 1)	
	SET @PageLowerBound = @PageSize * @PageIndex
	SET @PageUpperBound = @PageLowerBound + @PageSize + 1
	
	CREATE TABLE #DisplayOrderTmp 
	(
		[Id] int IDENTITY (1, 1) NOT NULL,
		[ProductId] int NOT NULL
	)

	SET @sql = '
	INSERT INTO #DisplayOrderTmp ([ProductId])
	SELECT p.Id
	FROM
		Product p with (NOLOCK)'
	
	IF @CategoryIdsCount > 0
	BEGIN
		SET @sql = @sql + '
		LEFT JOIN Product_Category_Mapping pcm with (NOLOCK)
			ON p.Id = pcm.ProductId'
	END
	
	IF @ManufacturerId > 0
	BEGIN
		SET @sql = @sql + '
		LEFT JOIN Product_Manufacturer_Mapping pmm with (NOLOCK)
			ON p.Id = pmm.ProductId'
	END


	SET @sql = @sql + '
	WHERE
		p.Deleted = 0 AND p.Published = 1 '
	
	--filter by category
	IF @CategoryIdsCount > 0
	BEGIN
		SET @sql = @sql + '
		AND pcm.CategoryId IN (SELECT CategoryId FROM #FilteredCategoryIds)'
		
		IF @FeaturedProducts IS NOT NULL
		BEGIN
			SET @sql = @sql + '
		AND pcm.IsFeaturedProduct = ' + CAST(@FeaturedProducts AS nvarchar(max))
		END
	END
	
	--filter by manufacturer
	IF @ManufacturerId > 0
	BEGIN
		SET @sql = @sql + '
		AND pmm.ManufacturerId = ' + CAST(@ManufacturerId AS nvarchar(max))
		
		IF @FeaturedProducts IS NOT NULL
		BEGIN
			SET @sql = @sql + '
		AND pmm.IsFeaturedProduct = ' + CAST(@FeaturedProducts AS nvarchar(max))
		END
	END
	
	--filter by vendor
	IF @VendorId > 0
	BEGIN
		SET @sql = @sql + '
		AND p.VendorId = ' + CAST(@VendorId AS nvarchar(max))
	END
	
	--filter by parent product identifer
	SET @sql = @sql + '
		AND p.VisibleIndividually = 1'

	
	--show hidden
	SET @sql = @sql + '
	AND (getutcdate() BETWEEN ISNULL(p.AvailableStartDateTimeUtc, ''1/1/1900'') and ISNULL(p.AvailableEndDateTimeUtc, ''1/1/2999''))'
	
	--min price
	IF @PriceMin > 0
	BEGIN
		SET @sql = @sql + '
		AND (p.Price >= ' + CAST(@PriceMin AS nvarchar(max)) + ')'
	END
	
	--max price
	IF @PriceMax > 0
	BEGIN
		SET @sql = @sql + '
		AND (p.Price <= ' + CAST(@PriceMax AS nvarchar(max)) + ')'
	END

	--show hidden and ACL
		SET @sql = @sql + '
		AND (p.SubjectToAcl = 0 OR EXISTS (
			SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
			WHERE
				[fcr].CustomerRoleId IN (
					SELECT [acl].CustomerRoleId
					FROM [AclRecord] acl with (NOLOCK)
					WHERE [acl].EntityId = p.Id AND [acl].EntityName = ''Product''
				)
			))'
	
	--show hidden and filter by store
	IF @StoreId > 0
	BEGIN
		SET @sql = @sql + '
		AND (p.LimitedToStores = 0 OR EXISTS (
			SELECT 1 FROM [StoreMapping] sm with (NOLOCK)
			WHERE [sm].EntityId = p.Id AND [sm].EntityName = ''Product'' and [sm].StoreId=' + CAST(@StoreId AS nvarchar(max)) + '
			))'
	END
	
	--filter by specs
	IF @SpecAttributesCount > 0
	BEGIN
		SET @sql = @sql + '
		AND NOT EXISTS (
			SELECT 1 FROM #FilteredSpecs [fs]
			WHERE
				[fs].SpecificationAttributeOptionId NOT IN (
					SELECT psam.SpecificationAttributeOptionId
					FROM Product_SpecificationAttribute_Mapping psam with (NOLOCK)
					WHERE psam.AllowFiltering = 1 AND psam.ProductId = p.Id
				)
			)'
	END
	
	--sorting
	SET @sql_orderby = ''	
	IF @OrderBy = 5 /* Name: A to Z */
		SET @sql_orderby = ' p.[Name] ASC'
	ELSE IF @OrderBy = 6 /* Name: Z to A */
		SET @sql_orderby = ' p.[Name] DESC'
	ELSE IF @OrderBy = 10 /* Price: Low to High */
		SET @sql_orderby = ' p.[Price] ASC'
	ELSE IF @OrderBy = 11 /* Price: High to Low */
		SET @sql_orderby = ' p.[Price] DESC'
	ELSE IF @OrderBy = 15 /* creation date */
		SET @sql_orderby = ' p.[CreatedOnUtc] DESC'
	ELSE /* default sorting, 0 (position) */
	BEGIN
		--category position (display order)
		IF @CategoryIdsCount > 0 SET @sql_orderby = ' pcm.DisplayOrder ASC'
		
		--manufacturer position (display order)
		IF @ManufacturerId > 0
		BEGIN
			IF LEN(@sql_orderby) > 0 SET @sql_orderby = @sql_orderby + ', '
			SET @sql_orderby = @sql_orderby + ' pmm.DisplayOrder ASC'
		END
		/*
		--parent grouped product specified (sort associated products)
		IF @ParentGroupedProductId > 0
		BEGIN
			IF LEN(@sql_orderby) > 0 SET @sql_orderby = @sql_orderby + ', '
			SET @sql_orderby = @sql_orderby + ' p.[DisplayOrder] ASC'
		END
		*/
		--name
		IF LEN(@sql_orderby) > 0 SET @sql_orderby = @sql_orderby + ', '
		SET @sql_orderby = @sql_orderby + ' p.[Name] ASC'
	END
	
	SET @sql = @sql + '
	ORDER BY' + @sql_orderby
	
	--PRINT (@sql)
	EXEC sp_executesql @sql

	DROP TABLE #FilteredCategoryIds
	DROP TABLE #FilteredSpecs

	CREATE TABLE #PageIndex 
	(
		[IndexId] int IDENTITY (1, 1) NOT NULL,
		[ProductId] int NOT NULL
	)
	INSERT INTO #PageIndex ([ProductId])
	SELECT ProductId
	FROM #DisplayOrderTmp
	GROUP BY ProductId
	ORDER BY min([Id])

	--total records
	SET @TotalRecords = @@rowcount
	
	DROP TABLE #DisplayOrderTmp

	--prepare filterable specification attribute option identifier (if requested)
	IF @LoadFilterableSpecificationAttributeOptionIds = 1
	BEGIN		
		CREATE TABLE #FilterableSpecs 
		(
			[SpecificationAttributeOptionId] int NOT NULL
		)
		INSERT INTO #FilterableSpecs ([SpecificationAttributeOptionId])
		SELECT DISTINCT [psam].SpecificationAttributeOptionId
		FROM [Product_SpecificationAttribute_Mapping] [psam] with (NOLOCK)
		WHERE [psam].[AllowFiltering] = 1
		AND [psam].[ProductId] IN (SELECT [pi].ProductId FROM #PageIndex [pi])

		--build comma separated list of filterable identifiers
		SELECT @FilterableSpecificationAttributeOptionIds = COALESCE(@FilterableSpecificationAttributeOptionIds + ',' , '') + CAST(SpecificationAttributeOptionId as nvarchar(4000))
		FROM #FilterableSpecs

		DROP TABLE #FilterableSpecs
 	END

	--return products
	create table #Products (Id int Not null, ProductTypeId int,ParentGroupedProductId int, Name nvarchar(400), ShortDescription nvarchar(max), FullDescription nvarchar(max),
		Price decimal(18, 4) NOT NULL, HasTierPrices bit NOT NULL, IndexId int Not null) 
	insert into #Products (Id,ProductTypeId,ParentGroupedProductId,Name,ShortDescription,FullDescription,
		Price,HasTierPrices, IndexId)
	select TOP (@RowsToReturn) P.Id,P.ProductTypeId,P.ParentGroupedProductId,P.Name,P.ShortDescription,P.FullDescription,
		P.Price,P.HasTierPrices,[pi].IndexId
	FROM
		#PageIndex [pi]
		INNER JOIN Product P with (NOLOCK) on P.Id = [pi].[ProductId]
	WHERE
		[pi].IndexId > @PageLowerBound AND 
		[pi].IndexId < @PageUpperBound
	ORDER BY
		[pi].IndexId
	
	DROP TABLE #PageIndex

		if exists(select * from #Products)
		begin
			if (@LanguageId>0)
			begin
				update #Products
				set Name=ltrim(rtrim(Substring(L.LocaleValue,1,400)))
				from #Products C, 
					LocalizedProperty L WITH (NOLOCK) 
				where C.Id=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='Product' and L.LocaleKey='Name'
			
				update #Products
				set ShortDescription=L.LocaleValue
				from #Products C, 
					LocalizedProperty L WITH (NOLOCK) 
				where C.Id=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='Product' and L.LocaleKey='ShortDescription'

				update #Products
				set FullDescription=L.LocaleValue
				from #Products C, 
					LocalizedProperty L WITH (NOLOCK) 
				where C.Id=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='Product' and L.LocaleKey='FullDescription'
			end
			
			if @IgnoreDiscounts=1
			begin
				delete from #ProductId
				insert into #ProductId (Id, ParentGroupedProductId, Price, HasTierPrices)
				select Id, ParentGroupedProductId, Price, HasTierPrices
				from #Products
				where ProductTypeId=5
				
				exec [FNS_CatalogModelFactory_ProductLoadAllPaged_GetMinimumPrice] @StoreId, 0
				
				update #Products
				set Price=P.Price
				from #Products F, #ProductId P
				where F.Id=P.Id
			end		
		
			--TierPrice
			insert into #TierPriceId (ProductId)
			select Id
			from #Products
			where HasTierPrices=1
						
			--UrlRecord SeName
			insert into #UrlRecord (ProductId,SeName)
			select TOP 1 WITH TIES U.EntityId as ProductId,ltrim(rtrim(Substring(U.Slug,1,400))) as SeName
			from UrlRecord U WITH (NOLOCK)
			where U.EntityId in (select Id from #Products) 
				and U.EntityName='Product' and U.LanguageId=0 and U.IsActive=1	
			ORDER BY ROW_NUMBER() OVER(PARTITION BY U.EntityId ORDER BY U.Id DESC)

			--Product Specification Attribute
			if @PrepareSpecificationAttributes=1
			begin
				insert into #ProductSpecificationAttributeMapping(Id)
				select Id 
				from Product_SpecificationAttribute_Mapping psm WITH (NOLOCK)
				where psm.ProductId in (select Id from #Products) 
			end
		end
		
	--#Products
	select F.Price, F.Name, F.ShortDescription, F.FullDescription, P.* 
	from Product P WITH(NOLOCK),#Products F
	where P.Id=F.Id
	order by F.IndexId
		
	--Group Product
	create table #GroupProducts (Id int Not null, ProductTypeId int,ParentGroupedProductId int, 
		Price decimal(18, 4) NOT NULL, HasTierPrices bit NOT NULL) 

	insert into #GroupProducts (Id,ProductTypeId,ParentGroupedProductId,Price,HasTierPrices)
	select P.Id,P.ProductTypeId,P.ParentGroupedProductId,P.Price,P.HasTierPrices
	from Product P With (NOLOCK)
	where P.ParentGroupedProductId in (select Id from #Products PG where PG.ProductTypeId=10)
		and P.Published=1 and P.Deleted=0
		and (P.SubjectToAcl = 0 OR EXISTS (
							SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
								WHERE
									[fcr].CustomerRoleId IN (
										SELECT [acl].CustomerRoleId
										FROM [AclRecord] acl
										WHERE [acl].EntityId = P.Id AND [acl].EntityName = 'Product'
									)
								))
		and (@StoreId=0 or P.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
				where sm.EntityName = 'Product' and sm.StoreId=@StoreId and sm.EntityId=P.Id))
	
	if @IgnoreDiscounts=1
	begin
		delete from #ProductId
		
		insert into #ProductId (Id, ParentGroupedProductId, Price, HasTierPrices)
		select P.Id,P.ParentGroupedProductId, P.Price,P.HasTierPrices
		from #GroupProducts P
		where P.ProductTypeId=5

		exec [FNS_CatalogModelFactory_ProductLoadAllPaged_GetMinimumPrice] @StoreId, 1
			
		delete from #GroupProducts
		where ProductTypeId=5 and Id not in (select Id from #ProductId)	
		
		update #GroupProducts
		set Price=P.Price
		from #GroupProducts F, #ProductId P
		where F.Id=P.Id
		
		delete from #ProductId
	end		

	select F.Price, P.* 
	from Product P WITH(NOLOCK),#GroupProducts F
	where P.Id=F.Id

	insert into #TierPriceId (ProductId)
	select Id
	from #GroupProducts
	where HasTierPrices=1
				
	--Product Specification Attribute
	if @PrepareSpecificationAttributes=1
	begin
		insert into #ProductSpecificationAttributeMapping(Id)
		select Id 
		from Product_SpecificationAttribute_Mapping psm WITH (NOLOCK)
		where psm.ProductId in (select Id from #GroupProducts) 
	end

	drop table #GroupProducts
	drop table #Products

	select @TotalRecords as Total,@FilterableSpecificationAttributeOptionIds as FilterableSpec 
end
--End Products

--UrlRecord
if @getFeaturedProducts=1 or @getProducts=1
begin	
	if (@LanguageId>0)
	begin
		update #UrlRecord
		set SeName=ltrim(rtrim(Substring(US.Slug,1,400)))
		from #UrlRecord C, (select TOP 1 WITH TIES U.EntityId,U.Slug
			from UrlRecord U WITH (NOLOCK)
			where U.EntityId in (select ProductId from #UrlRecord) 
				and U.EntityName='Product' and U.LanguageId=@LanguageId and U.IsActive=1
				and ltrim(rtrim(U.Slug))!=''
			ORDER BY ROW_NUMBER() OVER(PARTITION BY U.EntityId ORDER BY U.Id DESC)) US
		where C.ProductId=US.EntityId
	end			
	
	select * from  #UrlRecord
end
--end UrlRecord

--TierPrice
if @getFeaturedProducts=1 or @getProducts=1
begin
	select T.* 
	from TierPrice T WITH (NOLOCK)
	where T.ProductId in (select ProductId from #TierPriceId)
		and (T.StoreId=0 or T.StoreId=@StoreId) 
		and (T.CustomerRoleId is null or T.CustomerRoleId in (select CR.CustomerRoleId from #FilteredCustomerRoleIds CR))
		AND (getutcdate() BETWEEN ISNULL(T.StartDateTimeUtc, '1/1/1900') and ISNULL(T.EndDateTimeUtc, '1/1/2999'))
	order by T.ProductId,T.Quantity 
end
--end TierPrice				

--Product Specification Attribute
if @PrepareSpecificationAttributes=1
begin
	create table #ProductSpecificationModel (ProductId int, 
		SpecificationAttributeId int, 
		SpecificationAttributeOptionId int, 
		AttributeTypeId int, SpecificationAttributeName nvarchar(max), 
		ValueRaw nvarchar(max), ColorSquaresRgb nvarchar(100))

	insert into #ProductSpecificationModel (ProductId, SpecificationAttributeId, SpecificationAttributeOptionId, AttributeTypeId, SpecificationAttributeName, ValueRaw, ColorSquaresRgb)
	select psm.ProductId, sa.Id, psm.SpecificationAttributeOptionId, psm.AttributeTypeId, 
		sa.[Name], 
		ValueRaw=case
			when psm.AttributeTypeId=0 then sao.Name
			else psm.CustomValue
			end,
		sao.ColorSquaresRgb
	from Product_SpecificationAttribute_Mapping psm WITH (NOLOCK), 
		SpecificationAttributeOption sao WITH (NOLOCK), 
		SpecificationAttribute sa WITH (NOLOCK) 
	where psm.ShowOnProductPage=1 
		and psm.SpecificationAttributeOptionId=sao.Id
		and sao.SpecificationAttributeId = sa.Id
	order by psm.ProductId, psm.DisplayOrder, psm.Id

	if exists(select * from #ProductSpecificationModel) and @LanguageId>0
	begin
		update #ProductSpecificationModel
		set SpecificationAttributeName=ltrim(rtrim(Substring(L.LocaleValue,1,400)))
		from #ProductSpecificationModel C, 
			LocalizedProperty L WITH (NOLOCK) 
		where C.SpecificationAttributeId=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='SpecificationAttribute' and L.LocaleKey='Name'
			
		update #ProductSpecificationModel
		set ValueRaw=ltrim(rtrim(Substring(L.LocaleValue,1,400)))
		from #ProductSpecificationModel C, 
			LocalizedProperty L WITH (NOLOCK) 
		where C.AttributeTypeId=0 and 
			C.SpecificationAttributeOptionId=L.EntityId and L.LanguageId=@LanguageId and L.LocaleKeyGroup='SpecificationAttributeOption' and L.LocaleKey='Name'
	end
	select * from #ProductSpecificationModel
	drop table #ProductSpecificationModel
end
--end PrepareSpecificationAttributes
drop table #ProductId
drop table #TierPriceId
drop table #ProductSpecificationAttributeMapping
drop table #FilteredCustomerRoleIds
drop table #UrlRecord
	
END
GO
