
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetAllProductsDisplayedOnHomePage]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetAllProductsDisplayedOnHomePage]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetAllProductsDisplayedOnHomePage]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetAllProductsDisplayedOnHomePage]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetAllProductsDisplayedOnHomePage]
(
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@StoreId			int, -- Store mapping
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit --A value indicating whether to Ignore Store Limitations
)
AS
BEGIN
SET NOCOUNT ON
--filter by customer role IDs (access control list)
SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
CREATE TABLE #FilteredCustomerRoleIds
(
	CustomerRoleId int not null
)
INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

select P.*
from Product P  WITH (NOLOCK)
where P.Deleted=0
	and P.Published=1
	and P.ShowOnHomePage=1
	AND (getutcdate() BETWEEN ISNULL(P.AvailableStartDateTimeUtc, '1/1/1900') and ISNULL(P.AvailableEndDateTimeUtc, '1/1/2999'))
	and (@IgnoreAcl=1 or P.SubjectToAcl = 0 OR EXISTS (
					SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
						WHERE
							[fcr].CustomerRoleId IN (
								SELECT [acl].CustomerRoleId
								FROM [AclRecord] acl  WITH (NOLOCK)
								WHERE [acl].EntityId = P.Id AND [acl].EntityName = 'Product'
							)
						))
	and (@StoreId=0 or @IgnoreStoreLimitations=1 or P.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
		where sm.EntityName = 'Product' and sm.StoreId=@StoreId and sm.EntityId=P.Id))
Order by P.DisplayOrder,P.Id	
drop table #FilteredCustomerRoleIds
end
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetById]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetById]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetById]
(
	@productId int -- Product identifier
)
AS
BEGIN
SET NOCOUNT ON

select P.*
from Product P WITH (NOLOCK)
where P.Id=@productId
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetProductPicturesByProductId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetProductPicturesByProductId]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetProductPicturesByProductId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetProductPicturesByProductId]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetProductPicturesByProductId]
(
	@productId int -- Product identifier
)
AS
BEGIN
SET NOCOUNT ON
select P.*
from Product_Picture_Mapping P WITH (NOLOCK)
where P.ProductId=@productId
order by P.DisplayOrder, P.Id
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetProductPictureById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetProductPictureById]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetProductPictureById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetProductPictureById]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetProductPictureById]
(
	@productPictureId int -- Product picture identifier
)
AS
BEGIN
SET NOCOUNT ON
select P.*
from Product_Picture_Mapping P WITH (NOLOCK)
where P.Id=@productPictureId
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetTierPriceById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetTierPriceById]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetTierPriceById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetTierPriceById]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetTierPriceById]
(
	@tierPriceId int -- Tier price identifier
)
AS
BEGIN
SET NOCOUNT ON
select P.*
from TierPrice P WITH (NOLOCK)
where P.Id=@tierPriceId
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetCrossSellProductById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetCrossSellProductById]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetCrossSellProductById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetCrossSellProductById]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetCrossSellProductById]
(
	@crossSellProductId int -- Cross-sell product identifier
)
AS
BEGIN
SET NOCOUNT ON
select P.*
from CrossSellProduct P WITH (NOLOCK)
where P.Id=@crossSellProductId
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetRelatedProductById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetRelatedProductById]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetRelatedProductById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetRelatedProductById]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetRelatedProductById]
(
	@relatedProductId int -- Related product identifier
)
AS
BEGIN
SET NOCOUNT ON
select P.*
from RelatedProduct P WITH (NOLOCK)
where P.Id=@relatedProductId
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetProductBySku]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetProductBySku]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetProductBySku]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetProductBySku]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetProductBySku]
(
	@sku nvarchar(max) -- SKU
)
AS
BEGIN
SET NOCOUNT ON
select P.*
from Product P WITH (NOLOCK)
where P.Sku =@sku
	and P.Deleted=0
order by P.Id  	 
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetCrossSellProductsByProductId1]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetCrossSellProductsByProductId1]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetCrossSellProductsByProductId1]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetCrossSellProductsByProductId1]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetCrossSellProductsByProductId1]
(
	@productId1 int, -- The first product identifier
	@ShowHidden			bit --A value indicating whether to show hidden records
)
AS
BEGIN
SET NOCOUNT ON
select C.*
from CrossSellProduct C WITH (NOLOCK),
	Product P WITH (NOLOCK)
where C.ProductId2=P.Id
	and C.ProductId1=@productId1
	and P.Deleted=0
	and (P.Published=1 or @ShowHidden=1)
order by C.Id  	 
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetRelatedProductsByProductId1]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetRelatedProductsByProductId1]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetRelatedProductsByProductId1]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetRelatedProductsByProductId1]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetRelatedProductsByProductId1]
(
	@productId1 int, -- The first product identifier
	@ShowHidden			bit --A value indicating whether to show hidden records
)
AS
BEGIN
SET NOCOUNT ON
select Rp.*
from RelatedProduct Rp WITH (NOLOCK),
	Product P WITH (NOLOCK)
where Rp.ProductId2=P.Id
	and Rp.ProductId1=@productId1
	and P.Deleted=0
	and (P.Published=1 or @ShowHidden=1)
order by Rp.DisplayOrder, Rp.Id
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetProductsByIds]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetProductsByIds]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetProductsByIds]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetProductsByIds]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetProductsByIds]
(
	@commaSeparatedproductIds	nvarchar(MAX) = null	--a list of Product IDs (comma-separated list)
)
AS
BEGIN
SET NOCOUNT ON

--filter by customer role IDs (access control list)
SET @commaSeparatedproductIds = isnull(@commaSeparatedproductIds, '')	
CREATE TABLE #productIds
(
	orderid int identity(1,1),
	Id int not null
)
INSERT INTO #productIds (Id)
SELECT CAST(data as int) FROM [nop_splitstring_to_table](@commaSeparatedproductIds, ',')	
	
select P.*
from Product P WITH (NOLOCK), #productIds D
where P.Id=D.Id and P.Deleted=0
order by D.orderid

drop table #productIds
end
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetProductsByProductAtributeId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetProductsByProductAtributeId]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetProductsByProductAtributeId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetProductsByProductAtributeId]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetProductsByProductAtributeId]
(
	@productAttributeId int, --Product attribute identifier
	@PageIndex			int, --Page index
	@PageSize			int,  --Page size
	@TotalRecords		int OUTPUT
)
AS
BEGIN
SET NOCOUNT ON
--paging
DECLARE @PageLowerBound int
DECLARE @PageUpperBound int
DECLARE @RowsToReturn int
SET @RowsToReturn = @PageSize * (@PageIndex + 1)	
SET @PageLowerBound = @PageSize * @PageIndex
SET @PageUpperBound = @PageLowerBound + @PageSize + 1

CREATE TABLE #DisplayOrderTmp 
(
	[Id] int IDENTITY (1, 1) NOT NULL,
	[ProductId] int NOT NULL
)
	insert into #DisplayOrderTmp (ProductId)
	select P.Id  
	from Product P
	where P.Deleted=0
		and P.Id in (select distinct pam.ProductId from Product_ProductAttribute_Mapping pam   WITH (NOLOCK)
			where pam.ProductAttributeId = @productAttributeId)
	Order by P.Name
				
	CREATE TABLE #PageIndex 
	(
		[IndexId] int IDENTITY (1, 1) NOT NULL,
		[ProductId] int NOT NULL
	)
	INSERT INTO #PageIndex ([ProductId])
	SELECT [ProductId]
	FROM #DisplayOrderTmp
	GROUP BY [ProductId]
	ORDER BY min([Id])

	--total records
	SET @TotalRecords = @@rowcount
	
	DROP TABLE #DisplayOrderTmp

	--return products
	SELECT TOP (@RowsToReturn)
		P.*
	FROM
		#PageIndex [pi]
		INNER JOIN Product P on P.Id = [pi].[ProductId]
	WHERE
		[pi].IndexId > @PageLowerBound AND 
		[pi].IndexId < @PageUpperBound
	ORDER BY
		[pi].IndexId
	
	DROP TABLE #PageIndex
END
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Product_GetAssociatedProducts]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Product_GetAssociatedProducts]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Product_GetAssociatedProducts]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Product_GetAssociatedProducts]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[FNS_Product_GetAssociatedProducts]
(
	@parentGroupedProductId int, --Parent product identifier (used with grouped products)
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@StoreId			int, -- Store mapping
	@vendorId			int, -- Vendor mapping
	@ShowHidden			bit, --A value indicating whether to show hidden records
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit --A value indicating whether to Ignore Store Limitations
)
AS
BEGIN
SET NOCOUNT ON
if @ShowHidden=1
begin
	select P.*
	from Product P  WITH (NOLOCK)
	where P.ParentGroupedProductId=@parentGroupedProductId
		and P.Deleted=0
		and (@vendorId=0 or P.VendorId=@vendorId)
	Order by P.DisplayOrder, P.Id					
end
else
begin
	--filter by customer role IDs (access control list)
	SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
	CREATE TABLE #FilteredCustomerRoleIds
	(
		CustomerRoleId int not null
	)
	INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

	select P.*
	from Product P  WITH (NOLOCK)
	where P.ParentGroupedProductId=@parentGroupedProductId
		and P.Deleted=0
		and P.Published=1
		and (@vendorId=0 or P.VendorId=@vendorId)
		AND (getutcdate() BETWEEN ISNULL(P.AvailableStartDateTimeUtc, '1/1/1900') and ISNULL(P.AvailableEndDateTimeUtc, '1/1/2999'))
		and (@IgnoreAcl=1 or P.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl  WITH (NOLOCK)
									WHERE [acl].EntityId = P.Id AND [acl].EntityName = 'Product'
								)
							))
		and (@StoreId=0 or @IgnoreStoreLimitations=1 or P.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
			where sm.EntityName = 'Product' and sm.StoreId=@StoreId and sm.EntityId=P.Id))
	Order by P.DisplayOrder, P.Id	
	drop table #FilteredCustomerRoleIds
end                        
END
GO
