IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Category_GetAllCategories]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Category_GetAllCategories]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Category_GetAllCategories]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Category_GetAllCategories]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [FNS_Category_GetAllCategories]
(
	@categoryName nvarchar(MAX),	--Category name
	@StoreId			int, -- Store mapping
	@AllowedCustomerRoleIds	nvarchar(MAX) ,	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@PageIndex			int , --Page index
	@PageSize			int , --Page size
	@ShowHidden			bit , --A value indicating whether to show hidden records
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit, --A value indicating whether to Ignore Store Limitations
	@OverridePublished	bit = null, --null - process "Published" property according to "showHidden" parameter, true - load only "Published" products, false - load only "Unpublished" products
	@TotalRecords		int OUTPUT
)
AS
BEGIN
SET NOCOUNT ON

set @categoryName=ltrim(rtrim(isnull(@categoryName,'')))
--paging
DECLARE @PageLowerBound int
DECLARE @PageUpperBound int
DECLARE @RowsToReturn int
SET @RowsToReturn = @PageSize * (@PageIndex + 1)	
SET @PageLowerBound = @PageSize * @PageIndex
SET @PageUpperBound = @PageLowerBound + @PageSize + 1

CREATE TABLE #DisplayOrderTmp 
(
	[Id] int IDENTITY (1, 1) NOT NULL,
	[CategoryId] int NOT NULL
)

if @ShowHidden=1
begin
	insert into #DisplayOrderTmp (CategoryId)
	select C.Id  
	from Category C
	where (len(@categoryName)=0 or C.Name like '%'+@categoryName+'%')
		and C.Deleted=0
		and (@OverridePublished is null or C.Published = @OverridePublished)
	Order by C.ParentCategoryId,C.DisplayOrder,C.Id						
	
end
else
begin
	--filter by customer role IDs (access control list)
	SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
	CREATE TABLE #FilteredCustomerRoleIds
	(
		CustomerRoleId int not null
	)
	INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

	insert into #DisplayOrderTmp (CategoryId)
	select C.Id  
	from Category C
	where (len(@categoryName)=0 or C.Name like '%'+@categoryName+'%')
		and C.Published=1
		and C.Deleted=0
		and (@IgnoreAcl=1 or C.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl
									WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
								)
							))
		and (@StoreId=0 or @IgnoreStoreLimitations=1 or C.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
			where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))

	Order by C.ParentCategoryId,C.DisplayOrder,C.Id						

	drop table #FilteredCustomerRoleIds
end
				
	CREATE TABLE #PageIndex 
	(
		[IndexId] int IDENTITY (1, 1) NOT NULL,
		[CategoryId] int NOT NULL
	)
	INSERT INTO #PageIndex ([CategoryId])
	SELECT [CategoryId]
	FROM #DisplayOrderTmp
	GROUP BY [CategoryId]
	ORDER BY min([Id])

	--total records
	SET @TotalRecords = @@rowcount
	
	DROP TABLE #DisplayOrderTmp

	--return categories
	SELECT TOP (@RowsToReturn)
		C.*
	FROM
		#PageIndex [pi]
		INNER JOIN Category C on C.Id = [pi].[CategoryId]
	WHERE
		[pi].IndexId > @PageLowerBound AND 
		[pi].IndexId < @PageUpperBound
	ORDER BY
		[pi].IndexId
	
	DROP TABLE #PageIndex
END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Category_GetAllCategoriesByParentCategoryId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Category_GetAllCategoriesByParentCategoryId]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Category_GetAllCategoriesByParentCategoryId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Category_GetAllCategoriesByParentCategoryId]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [FNS_Category_GetAllCategoriesByParentCategoryId]
(
	@parentCategoryId int,	--Parent category identifier
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@StoreId			int, -- Store mapping
	@ShowHidden			bit, --A value indicating whether to show hidden records
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit, --A value indicating whether to Ignore Store Limitations
	@includeAllLevels bit --A value indicating whether we should load all child levels
)
AS
BEGIN
SET NOCOUNT ON
create table #tmpCategory (Id int,Levelsort int)

if @ShowHidden=1
begin
	insert into #tmpCategory (Id, Levelsort)
	select C.Id, 0
	from Category C  WITH (NOLOCK)
	where C.ParentCategoryId=@parentCategoryId
		and C.Deleted=0
	Order by C.DisplayOrder, C.Id
end
else
begin
	--filter by customer role IDs (access control list)
	SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
	CREATE TABLE #FilteredCustomerRoleIds
	(
		CustomerRoleId int not null
	)
	INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

	insert into #tmpCategory (Id, Levelsort)
	select C.Id, 0
	from Category C  WITH (NOLOCK)
	where C.ParentCategoryId=@parentCategoryId
		and C.Deleted=0
		and C.Published=1
		and (@IgnoreAcl=1 or C.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl  WITH (NOLOCK)
									WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
								)
							))
		and (@StoreId=0 or @IgnoreStoreLimitations=1 or C.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
			where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))
	Order by C.DisplayOrder, C.Id	
	drop table #FilteredCustomerRoleIds
end

if @includeAllLevels=1
begin
	--we must find all categories
	create table #CategoryId (Id int,ParentCategoryId int,SubjectToAcl bit,LimitedToStores bit, LevelSort int)

	--Category
	;with cte_tree (Id, ParentCategoryId, SubjectToAcl, LimitedToStores, LevelSort) as 
	(
		select C.Id, C.ParentCategoryId, C.SubjectToAcl, C.LimitedToStores, 0 as LevelSort
		from Category C WITH (NOLOCK)
		where C.Id in (select Id from #tmpCategory)
			and C.Deleted=0
			and C.Published=1
			union all
		select T.Id, T.ParentCategoryId, T.SubjectToAcl, T.LimitedToStores, C.LevelSort+1 as LevelSort 
		from cte_tree C,Category T WITH (NOLOCK)
		where C.Id=T.ParentCategoryId
			and T.Deleted=0
			and T.Published=1
	)
		
	insert into #CategoryId (Id, ParentCategoryId, SubjectToAcl, LimitedToStores, LevelSort)			
	select Id, ParentCategoryId, SubjectToAcl, LimitedToStores, LevelSort
	from cte_tree

	--ACL Category
	if @ShowHidden=0 and @IgnoreAcl=0 and exists(select Id from #CategoryId where SubjectToAcl=1)		
	begin
		delete #CategoryId
		from #CategoryId C
		where C.SubjectToAcl=1 and not EXISTS (
					SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
								SELECT [acl].CustomerRoleId
								FROM [AclRecord] acl  WITH (NOLOCK)
								WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
						)
					)
	end

	--StoreMapping Category
	if @ShowHidden=0 
		and @StoreId>0
		and @IgnoreStoreLimitations=0 
		and exists(select Id from #CategoryId where LimitedToStores=1)		
	begin
		delete #CategoryId
		from #CategoryId C
		where C.LimitedToStores=1 and not Exists(select Top 1 Id from StoreMapping sm WITH (NOLOCK)
					where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id)
	end

	while (1=1)
	begin
		delete #CategoryId
		from #CategoryId C
		where C.ParentCategoryId!=0 
			and C.ParentCategoryId not in (select Id from #CategoryId)
		if @@ROWCOUNT=0
			break
	end

	truncate table #tmpCategory

	insert into #tmpCategory (Id, Levelsort)
	select Id, Levelsort
	from #CategoryId
end

select C.*
from Category C  WITH (NOLOCK), #tmpCategory S
where C.Id=S.Id
Order by S.Levelsort, C.DisplayOrder, C.Id		

drop table #tmpCategory

END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Category_GetAllCategoriesDisplayedOnHomePage]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Category_GetAllCategoriesDisplayedOnHomePage]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Category_GetAllCategoriesDisplayedOnHomePage]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Category_GetAllCategoriesDisplayedOnHomePage]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [FNS_Category_GetAllCategoriesDisplayedOnHomePage]
(
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@StoreId			int, -- Store mapping
	@ShowHidden			bit, --A value indicating whether to show hidden records
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit --A value indicating whether to Ignore Store Limitations
)
AS
BEGIN
SET NOCOUNT ON
if @ShowHidden=1
begin
	select C.*
	from Category C  WITH (NOLOCK)
	where C.Deleted=0
		and C.Published=1
		and C.ShowOnHomePage=1
	Order by C.DisplayOrder						
end
else
begin
	--filter by customer role IDs (access control list)
	SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
	CREATE TABLE #FilteredCustomerRoleIds
	(
		CustomerRoleId int not null
	)
	INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

	select C.*
	from Category C  WITH (NOLOCK)
	where C.ShowOnHomePage=1
		and C.Deleted=0
		and C.Published=1
		and (@IgnoreAcl=1 or C.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl  WITH (NOLOCK)
									WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
								)
							))
		and (@StoreId=0 or @IgnoreStoreLimitations=1 or C.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
			where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))
	Order by C.DisplayOrder, C.Id	
	drop table #FilteredCustomerRoleIds
end                        
END
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Category_GetById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Category_GetById]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Category_GetById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Category_GetById]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Category_GetProductCategoriesByCategoryId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Category_GetProductCategoriesByCategoryId]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Category_GetProductCategoriesByCategoryId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Category_GetProductCategoriesByCategoryId]
GO
/****** Object:  StoredProcedure [FNS_Category_GetAllCategories]    Script Date: 04/03/2013 09:44:23 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [FNS_Category_GetProductCategoriesByCategoryId]
(
	@categoryId int ,	--Category identifier
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@PageIndex			int , --Page index
	@PageSize			int, --Page size
	@StoreId			int, -- Store mapping
	@ShowHidden			bit , --A value indicating whether to show hidden records
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit, --A value indicating whether to Ignore Store Limitations
	@TotalRecords		int OUTPUT
)
AS
BEGIN
SET NOCOUNT ON

--paging
DECLARE @PageLowerBound int
DECLARE @PageUpperBound int
DECLARE @RowsToReturn int
SET @RowsToReturn = @PageSize * (@PageIndex + 1)	
SET @PageLowerBound = @PageSize * @PageIndex
SET @PageUpperBound = @PageLowerBound + @PageSize + 1

CREATE TABLE #DisplayOrderTmp 
(
	[Id] int IDENTITY (1, 1) NOT NULL,
	[CategoryProductId] int NOT NULL
)

if @ShowHidden=1
begin
	insert into #DisplayOrderTmp (CategoryProductId)
	select C.Id  
	from Product_Category_Mapping C  WITH (NOLOCK),Product P  WITH (NOLOCK)
	where C.CategoryId=@categoryId
		and C.ProductId=P.Id
		and P.Deleted=0
	Order by C.DisplayOrder, C.Id
end
else
begin
	--filter by customer role IDs (access control list)
	SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
	CREATE TABLE #FilteredCustomerRoleIds
	(
		CustomerRoleId int not null
	)
	INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

	insert into #DisplayOrderTmp (CategoryProductId)
	select C.Id  
	from Product_Category_Mapping C  WITH (NOLOCK),Product P  WITH (NOLOCK),Category Ct  WITH (NOLOCK)
	where C.CategoryId=@categoryId
		and C.ProductId=P.Id
		and P.Deleted=0
		and P.Published=1
		and Ct.Id=@categoryId
		and (@IgnoreAcl=1 or Ct.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl  WITH (NOLOCK)
									WHERE [acl].EntityId = Ct.Id AND [acl].EntityName = 'Category'
								)
							))
		and (@StoreId=0 or @IgnoreStoreLimitations=1 or Ct.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
			where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=Ct.Id))
	Order by C.DisplayOrder, C.Id	
	drop table #FilteredCustomerRoleIds
end
					
	CREATE TABLE #PageIndex 
	(
		[IndexId] int IDENTITY (1, 1) NOT NULL,
		[CategoryProductId] int NOT NULL
	)
	INSERT INTO #PageIndex (CategoryProductId)
	SELECT CategoryProductId
	FROM #DisplayOrderTmp
	GROUP BY CategoryProductId
	ORDER BY min([Id])

	--total records
	SET @TotalRecords = @@rowcount
	
	DROP TABLE #DisplayOrderTmp

	--return categories
	SELECT TOP (@RowsToReturn)
		C.*
	FROM
		#PageIndex [pi]
		INNER JOIN Product_Category_Mapping C  WITH (NOLOCK) on C.Id = [pi].[CategoryProductId]
	WHERE
		[pi].IndexId > @PageLowerBound AND 
		[pi].IndexId < @PageUpperBound
	ORDER BY
		[pi].IndexId
	
	DROP TABLE #PageIndex
END

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Category_GetProductCategoriesByProductId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Category_GetProductCategoriesByProductId]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Category_GetProductCategoriesByProductId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Category_GetProductCategoriesByProductId]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [FNS_Category_GetProductCategoriesByProductId]
(
	@productId int ,	--Product identifier
	@AllowedCustomerRoleIds	nvarchar(MAX) ,	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@StoreId			int, -- Store mapping
	@ShowHidden			bit, --A value indicating whether to show hidden records
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit --A value indicating whether to Ignore Store Limitations
)
AS
BEGIN
SET NOCOUNT ON

if @ShowHidden=1
begin
	select Pc.*
	from Product_Category_Mapping Pc WITH (NOLOCK), Category C  WITH (NOLOCK)
	where Pc.CategoryId=C.Id 
		and  Pc.ProductId=@productId
		and C.Deleted=0
	Order by Pc.DisplayOrder, Pc.Id
end
else
begin
	--filter by customer role IDs (access control list)
	SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
	CREATE TABLE #FilteredCustomerRoleIds
	(
		CustomerRoleId int not null
	)
	INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
	SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

	select Pc.*
	from Product_Category_Mapping Pc  WITH (NOLOCK),Category C  WITH (NOLOCK)
	where Pc.CategoryId=C.Id 
		and  Pc.ProductId=@productId
		and C.Deleted=0
		and C.Published=1
		and (@IgnoreAcl=1 or C.SubjectToAcl = 0 OR EXISTS (
						SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
							WHERE
								[fcr].CustomerRoleId IN (
									SELECT [acl].CustomerRoleId
									FROM [AclRecord] acl  WITH (NOLOCK)
									WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
								)
							))
		and (@StoreId=0 or @IgnoreStoreLimitations=1 or C.LimitedToStores=0 or Exists(select Top 1 Id from StoreMapping sm   WITH (NOLOCK)
			where sm.EntityName = 'Category' and sm.StoreId=@StoreId and sm.EntityId=C.Id))

	Order by Pc.DisplayOrder	
	
	drop table #FilteredCustomerRoleIds
end

END
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_Category_ProductCategoryGetById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_Category_ProductCategoryGetById]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FNS_Category_ProductCategoryGetById]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FNS_Category_ProductCategoryGetById]
GO
