IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_OrderReportService_GetAlsoPurchasedProductsIds]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_OrderReportService_GetAlsoPurchasedProductsIds]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [FNS_OrderReportService_GetAlsoPurchasedProductsIds]
(
	@productId int, -- Product identifier
	@storeId int, -- Store identifier
	@recordsToReturn int, -- Records to return
	@visibleIndividuallyOnly bit, -- A values indicating whether to load only products marked as "visible individually"; "false" to load all records; "true" to load "visible individually" only
	@showHidden bit -- A value indicating whether to show hidden records
)
AS
BEGIN
SET NOCOUNT ON
	if @recordsToReturn<=0
		set @recordsToReturn=1000

	select top (@recordsToReturn) oi.ProductId as ProductId, SUM(oi.Quantity) as Quantity 
	from [OrderItem] oi WITH (NOLOCK),[Order] oo WITH (NOLOCK)
	where oi.OrderId=oo.Id 
		and oo.Id in (select OrderId from [OrderItem] WITH (NOLOCK) where ProductId=@productId)
		and oi.ProductId!=@productId
		and (@storeId=0 or oo.StoreId=@storeId)
		and oo.Deleted=0
		and oi.ProductId in (select Id from Product p WITH (NOLOCK) where p.Deleted=0
													and (@showHidden=1 or p.Published=1)
													and (@visibleIndividuallyOnly=0 or p.VisibleIndividually=1))
	group by oi.ProductId
	order by 2 desc
end
GO